@@ -3,6 +3,7 @@
    Protocol services - Virtual channels
    Copyright 2003 Erik Forsberg <forsberg@cendio.se> for Cendio AB
    Copyright (C) Matthew Chapman <matthewc.unsw.edu.au> 2003-2008
+   Copyright 2016 Alexander Zakharov <uglym8@gmail.com>
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -29,6 +30,8 @@
 extern RDP_VERSION g_rdp_version;
 extern RD_BOOL g_encryption;
 
+uint32 vc_chunk_size = CHANNEL_CHUNK_LENGTH;
+
 VCHANNEL g_channels[MAX_CHANNELS];
 unsigned int g_num_channels;
 
@@ -52,7 +55,8 @@ channel_register(char *name, uint32 flags, void (*callback) (STREAM))
 
 	if (g_num_channels >= MAX_CHANNELS)
 	{
-		error("Channel table full, increase MAX_CHANNELS\n");
+		logger(Core, Error,
+		       "channel_register(), channel table full, increase MAX_CHANNELS");
 		return NULL;
 	}
 
@@ -68,6 +72,7 @@ channel_register(char *name, uint32 flags, void (*callback) (STREAM))
 STREAM
 channel_init(VCHANNEL * channel, uint32 length)
 {
+	UNUSED(channel);
 	STREAM s;
 
 	s = sec_init(g_encryption ? SEC_ENCRYPT : 0, length + 8);
@@ -75,58 +80,101 @@ channel_init(VCHANNEL * channel, uint32 length)
 	return s;
 }
 
+static void
+channel_send_chunk(STREAM s, VCHANNEL * channel, uint32 length)
+{
+	uint32 flags;
+	uint32 thislength;
+	RD_BOOL inplace;
+	STREAM chunk;
+
+	/* Note: In the original clipboard implementation, this number was
+	   1592, not 1600. However, I don't remember the reason and 1600 seems
+	   to work so.. This applies only to *this* length, not the length of
+	   continuation or ending packets. */
+
+	/* Actually, CHANNEL_CHUNK_LENGTH (default value is 1600 bytes) is described
+	   in MS-RDPBCGR (s. 2.2.6, s.3.1.5.2.1) and can be set by server only
+	   in the optional field VCChunkSize of VC Caps) */
+
+	thislength = MIN(s_remaining(s), vc_chunk_size);
+
+	flags = 0;
+	if (length == s_remaining(s))
+	{
+		flags |= CHANNEL_FLAG_FIRST;
+	}
+	if (s_remaining(s) == thislength)
+	{
+		flags |= CHANNEL_FLAG_LAST;
+	}
+	if (channel->flags & CHANNEL_OPTION_SHOW_PROTOCOL)
+	{
+		flags |= CHANNEL_FLAG_SHOW_PROTOCOL;
+	}
+
+	logger(Protocol, Debug, "channel_send_chunk(), sending %d bytes with flags 0x%x",
+	       thislength, flags);
+
+	/* first fragment sent in-place */
+	inplace = False;
+	if ((flags & (CHANNEL_FLAG_FIRST|CHANNEL_FLAG_LAST)) ==
+	    (CHANNEL_FLAG_FIRST|CHANNEL_FLAG_LAST))
+	{
+		inplace = True;
+	}
+
+	if (inplace)
+	{
+		s_pop_layer(s, channel_hdr);
+		chunk = s;
+	}
+	else
+	{
+		chunk = sec_init(g_encryption ? SEC_ENCRYPT : 0, thislength + 8);
+	}
+
+	out_uint32_le(chunk, length);
+	out_uint32_le(chunk, flags);
+	if (!inplace)
+	{
+		out_uint8stream(chunk, s, thislength);
+		s_mark_end(chunk);
+	}
+	sec_send_to_channel(chunk, g_encryption ? SEC_ENCRYPT : 0, channel->mcs_id);
+
+	/* Sending modifies the current offset, so make it is marked as
+	   fully completed. */
+	if (inplace)
+	{
+		in_uint8s(s, s_remaining(s));
+	}
+
+	if (!inplace)
+	{
+		s_free(chunk);
+	}
+}
+
 void
 channel_send(STREAM s, VCHANNEL * channel)
 {
-	uint32 length, flags;
-	uint32 thislength, remaining;
-	uint8 *data;
+	uint32 length;
 
 #ifdef WITH_SCARD
 	scard_lock(SCARD_LOCK_CHANNEL);
 #endif
 
-	/* first fragment sent in-place */
 	s_pop_layer(s, channel_hdr);
-	length = s->end - s->p - 8;
+	in_uint8s(s, 8);
+	length = s_remaining(s);
 
-	DEBUG_CHANNEL(("channel_send, length = %d\n", length));
+	logger(Protocol, Debug, "channel_send(), channel = %d, length = %d", channel->mcs_id,
+	       length);
 
-	thislength = MIN(length, CHANNEL_CHUNK_LENGTH);
-/* Note: In the original clipboard implementation, this number was
-   1592, not 1600. However, I don't remember the reason and 1600 seems
-   to work so.. This applies only to *this* length, not the length of
-   continuation or ending packets. */
-	remaining = length - thislength;
-	flags = (remaining == 0) ? CHANNEL_FLAG_FIRST | CHANNEL_FLAG_LAST : CHANNEL_FLAG_FIRST;
-	if (channel->flags & CHANNEL_OPTION_SHOW_PROTOCOL)
-		flags |= CHANNEL_FLAG_SHOW_PROTOCOL;
-
-	out_uint32_le(s, length);
-	out_uint32_le(s, flags);
-	data = s->end = s->p + thislength;
-	DEBUG_CHANNEL(("Sending %d bytes with FLAG_FIRST\n", thislength));
-	sec_send_to_channel(s, g_encryption ? SEC_ENCRYPT : 0, channel->mcs_id);
-
-	/* subsequent segments copied (otherwise would have to generate headers backwards) */
-	while (remaining > 0)
+	while (!s_check_end(s))
 	{
-		thislength = MIN(remaining, CHANNEL_CHUNK_LENGTH);
-		remaining -= thislength;
-		flags = (remaining == 0) ? CHANNEL_FLAG_LAST : 0;
-		if (channel->flags & CHANNEL_OPTION_SHOW_PROTOCOL)
-			flags |= CHANNEL_FLAG_SHOW_PROTOCOL;
-
-		DEBUG_CHANNEL(("Sending %d bytes with flags %d\n", thislength, flags));
-
-		s = sec_init(g_encryption ? SEC_ENCRYPT : 0, thislength + 8);
-		out_uint32_le(s, length);
-		out_uint32_le(s, flags);
-		out_uint8p(s, data, thislength);
-		s_mark_end(s);
-		sec_send_to_channel(s, g_encryption ? SEC_ENCRYPT : 0, channel->mcs_id);
-
-		data += thislength;
+		channel_send_chunk(s, channel, length);
 	}
 
 #ifdef WITH_SCARD
@@ -166,22 +214,17 @@ channel_process(STREAM s, uint16 mcs_channel)
 		in = &channel->in;
 		if (flags & CHANNEL_FLAG_FIRST)
 		{
-			if (length > in->size)
-			{
-				in->data = (uint8 *) xrealloc(in->data, length);
-				in->size = length;
-			}
-			in->p = in->data;
+			s_realloc(in, length);
+			s_reset(in);
 		}
 
-		thislength = MIN(s->end - s->p, in->data + in->size - in->p);
-		memcpy(in->p, s->p, thislength);
-		in->p += thislength;
+		thislength = s_remaining(s);
+		out_uint8stream(in, s, thislength);
 
 		if (flags & CHANNEL_FLAG_LAST)
 		{
-			in->end = in->p;
-			in->p = in->data;
+			s_mark_end(in);
+			s_seek(in, 0);
 			channel->process(in);
 		}
 	}
