@@ -2,6 +2,8 @@
    rdesktop: A Remote Desktop Protocol client.
    Miscellaneous protocol constants
    Copyright (C) Matthew Chapman 1999-2008
+   Copyright 2016 Alexander Zakharov <uglym8@gmail.com>
+   Copyright 2017-2018 Henrik Andersson <hean01@cendio.se> for Cendio AB
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -17,12 +19,48 @@
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
+#ifndef _CONSTANTS_H
+#define _CONSTANTS_H
+
 /* TCP port for Remote Desktop Protocol */
 #define TCP_PORT_RDP 3389
 
 #define DEFAULT_CODEPAGE	"UTF-8"
 #define WINDOWS_CODEPAGE	"UTF-16LE"
 
+/* T-REC-T.123-200701, section 8 */
+#define T123_HEADER_VERSION 0x3
+
+/* [MS-RDPBCGR] 2.2.9.1.2 */
+#define FASTPATH_OUTPUT_ACTION_FASTPATH	0x0
+#define FASTPATH_OUTPUT_ACTION_X224	T123_HEADER_VERSION
+
+#define FASTPATH_OUTPUT_SECURE_CHECKSUM 0x1
+#define FASTPATH_OUTPUT_ENCRYPTED       0x2
+
+/* [MS-RDPBCGR] 2.2.9.1.2.1 */
+/* adjusted for position in updateHeader */
+#define FASTPATH_UPDATETYPE_ORDERS		0x0
+#define FASTPATH_UPDATETYPE_BITMAP		0x1
+#define FASTPATH_UPDATETYPE_PALETTE		0x2
+#define FASTPATH_UPDATETYPE_SYNCHRONIZE		0x3
+#define FASTPATH_UPDATETYPE_SURFCMDS		0x4
+#define FASTPATH_UPDATETYPE_PTR_NULL		0x5
+#define FASTPATH_UPDATETYPE_PTR_DEFAULT		0x6
+#define FASTPATH_UPDATETYPE_PTR_POSITION	0x8
+#define FASTPATH_UPDATETYPE_COLOR		0x9
+#define FASTPATH_UPDATETYPE_CACHED		0xA
+#define FASTPATH_UPDATETYPE_POINTER		0xB
+
+#define FASTPATH_FRAGMENT_SINGLE	(0x0 << 4)
+#define FASTPATH_FRAGMENT_LAST		(0x1 << 4)
+#define FASTPATH_FRAGMENT_FIRST		(0x2 << 4)
+#define FASTPATH_FRAGMENT_NEXT		(0x3 << 4)
+
+#define FASTPATH_OUTPUT_COMPRESSION_USED	(0x2 << 6)
+
+#define RDESKTOP_FASTPATH_MULTIFRAGMENT_MAX_SIZE 65535
+
 /* ISO PDU codes */
 enum ISO_PDU_CODE
 {
@@ -87,6 +125,17 @@ enum MCS_PDU_TYPE
 #define MCS_GLOBAL_CHANNEL	1003
 #define MCS_USERCHANNEL_BASE    1001
 
+/* ITU-T Rec. T.125, Reason enumeration used with Disconnect Provider
+   Ultimatum, see mcs_send_dpu(reason) */
+enum MCS_DPU_REASON
+{
+	RN_DOMAIN_DISCONNECTED = 0,
+	RN_PROVIDER_INITIATED,
+	RN_TOKEN_PURGED,
+	RN_USER_REQUESTED,
+	RN_CHANNEL_PURGED,
+};
+
 /* RDP secure transport constants */
 #define SEC_RANDOM_SIZE		32
 #define SEC_MODULUS_SIZE	64
@@ -94,20 +143,32 @@ enum MCS_PDU_TYPE
 #define SEC_PADDING_SIZE	8
 #define SEC_EXPONENT_SIZE	4
 
-#define SEC_CLIENT_RANDOM	0x0001
+/* TS_SECURITY_HEADER.flags */
+#define SEC_EXCHANGE_PKT	0x0001
+#define SEC_TRANSPORT_REQ	0x0002
+#define RDP_SEC_TRANSPORT_RSP	0x0004
 #define SEC_ENCRYPT		0x0008
-#define SEC_LOGON_INFO		0x0040
-#define SEC_LICENCE_NEG		0x0080
-#define SEC_REDIRECT_ENCRYPT	0x0C00
+#define SEC_RESET_SEQNO		0x0010
+#define SEC_IGNORE_SEQNO	0x0020
+#define SEC_INFO_PKT		0x0040
+#define SEC_LICENSE_PKT		0x0080
+#define SEC_LICENSE_ENCRYPT_CS	0x0200
+#define SEC_LICENSE_ENCRYPT_SC	0x0200
+#define SEC_REDIRECTION_PKT	0x0400
+#define SEC_SECURE_CHECKSUM	0x0800
+#define SEC_AUTODETECT_REQ	0x1000
+#define SEC_AUTODETECT_RSP	0x2000
+#define SEC_HEARTBEAT		0x4000
+#define SEC_FLAGSHI_VALID	0x8000
 
 #define SEC_TAG_SRV_INFO	0x0c01
 #define SEC_TAG_SRV_CRYPT	0x0c02
 #define SEC_TAG_SRV_CHANNELS	0x0c03
 
-#define SEC_TAG_CLI_INFO	0xc001
-#define SEC_TAG_CLI_CRYPT	0xc002
-#define SEC_TAG_CLI_CHANNELS    0xc003
-#define SEC_TAG_CLI_CLUSTER     0xc004
+#define CS_CORE			0xc001
+#define CS_SECURITY		0xc002
+#define CS_NET			0xc003
+#define CS_CLUSTER		0xc004
 
 #define SEC_TAG_PUBKEY		0x0006
 #define SEC_TAG_KEYSIG		0x0008
@@ -131,12 +192,12 @@ enum MCS_PDU_TYPE
 #define LICENCE_SIGNATURE_SIZE	16
 
 #define LICENCE_TAG_REQUEST                     0x01
-#define LICENCE_TAG_PLATFORM_CHALLANGE          0x02
+#define LICENCE_TAG_PLATFORM_CHALLENGE          0x02
 #define LICENCE_TAG_NEW_LICENCE                 0x03
 #define LICENCE_TAG_UPGRADE_LICENCE             0x04
 #define LICENCE_TAG_LICENCE_INFO                0x12
 #define LICENCE_TAG_NEW_LICENCE_REQUEST         0x13
-#define LICENCE_TAG_PLATFORM_CHALLANGE_RESPONSE 0x15
+#define LICENCE_TAG_PLATFORM_CHALLENGE_RESPONSE 0x15
 #define LICENCE_TAG_ERROR_ALERT                 0xff
 
 #define BB_CLIENT_USER_NAME_BLOB	0x000f
@@ -155,18 +216,18 @@ enum RDP_PDU_TYPE
 
 enum RDP_DATA_PDU_TYPE
 {
-	RDP_DATA_PDU_UPDATE = 2,
-	RDP_DATA_PDU_CONTROL = 20,
-	RDP_DATA_PDU_POINTER = 27,
-	RDP_DATA_PDU_INPUT = 28,
-	RDP_DATA_PDU_SYNCHRONISE = 31,
-	RDP_DATA_PDU_BELL = 34,
-	RDP_DATA_PDU_CLIENT_WINDOW_STATUS = 35,
-	RDP_DATA_PDU_LOGON = 38,	/* PDUTYPE2_SAVE_SESSION_INFO */
-	RDP_DATA_PDU_FONT2 = 39,
-	RDP_DATA_PDU_KEYBOARD_INDICATORS = 41,
-	RDP_DATA_PDU_DISCONNECT = 47,
-	RDP_DATA_PDU_AUTORECONNECT_STATUS = 50
+	RDP_DATA_PDU_UPDATE = 0x02,	/* PDUTYPE2_UPDATE */
+	RDP_DATA_PDU_CONTROL = 0x14,	/* PDUTYPE2_CONTROL */
+	RDP_DATA_PDU_POINTER = 0x1b,	/* PDUTYPE2_POINTER */
+	RDP_DATA_PDU_INPUT = 0x1c,	/* PDUTYPE2_INPUT */
+	RDP_DATA_PDU_SYNCHRONISE = 0x1f,	/* PDUTYPE2_SYNCHRONIZE */
+	RDP_DATA_PDU_BELL = 0x22,	/* PDUTYPE2_PLAY_SOUND */
+	RDP_DATA_PDU_CLIENT_WINDOW_STATUS = 0x23,	/* PDUTYPE2_SUPRESS_OUTPUT */
+	RDP_DATA_PDU_LOGON = 0x26,	/* PDUTYPE2_SAVE_SESSION_INFO */
+	RDP_DATA_PDU_FONT2 = 0x27,	/* PDUTYPE2_FONTLIST */
+	RDP_DATA_PDU_KEYBOARD_INDICATORS = 0x29,	/* PDUTYPE2_SET_KEYBOARD_INDICATORS */
+	RDP_DATA_PDU_SET_ERROR_INFO = 0x2f,	/* PDUTYPE2_SET_ERROR_INFO */
+	RDP_DATA_PDU_AUTORECONNECT_STATUS = 0x32,	/* PDUTYPE2_ARC_STATUS_PDU */
 };
 
 enum RDP_SAVE_SESSION_PDU_TYPE
@@ -208,10 +269,11 @@ enum RDP_POINTER_PDU_TYPE
 	RDP_POINTER_NEW = 8
 };
 
+/* [MS-RDPBCGR] 2.2.9.1.1.4.3 */
 enum RDP_SYSTEM_POINTER_TYPE
 {
-	RDP_NULL_POINTER = 0,
-	RDP_DEFAULT_POINTER = 0x7F00
+	SYSPTR_NULL = 0x00000000,
+	SYSPTR_DEFAULT = 0x00007F00
 };
 
 enum RDP_INPUT_DEVICE
@@ -220,12 +282,14 @@ enum RDP_INPUT_DEVICE
 	RDP_INPUT_CODEPOINT = 1,
 	RDP_INPUT_VIRTKEY = 2,
 	RDP_INPUT_SCANCODE = 4,
-	RDP_INPUT_MOUSE = 0x8001
+	RDP_INPUT_MOUSE = 0x8001,
+	RDP_INPUT_MOUSEX = 0x8002
 };
 
 /* Device flags */
 #define KBD_FLAG_RIGHT          0x0001
 #define KBD_FLAG_EXT            0x0100
+#define KBD_FLAG_EXT1           0x0200
 #define KBD_FLAG_QUIET          0x1000
 #define KBD_FLAG_DOWN           0x4000
 #define KBD_FLAG_UP             0x8000
@@ -245,6 +309,8 @@ enum RDP_INPUT_DEVICE
 #define MOUSE_FLAG_BUTTON3      0x4000
 #define MOUSE_FLAG_BUTTON4      0x0280
 #define MOUSE_FLAG_BUTTON5      0x0380
+#define MOUSEX_FLAG_BUTTON1     0x0001
+#define MOUSEX_FLAG_BUTTON2     0x0002
 #define MOUSE_FLAG_DOWN         0x8000
 
 /* Raster operation masks */
@@ -308,13 +374,34 @@ enum RDP_INPUT_DEVICE
 #define RDP_CAPSET_COLCACHE	10
 #define RDP_CAPLEN_COLCACHE	0x08
 
+#define RDP_CAPSET_SOUND	12
+#define RDP_CAPLEN_SOUND	8
+
+#define RDP_CAPSET_INPUT	13
+#define RDP_CAPLEN_INPUT	88
+
+#define RDP_CAPSET_FONT		14
+#define RDP_CAPLEN_FONT		8
+
 #define RDP_CAPSET_BRUSHCACHE	15
 #define RDP_CAPLEN_BRUSHCACHE	0x08
 
+#define RDP_CAPSET_GLYPHCACHE	16
+#define RDP_CAPLEN_GLYPHCACHE	52
+
 #define RDP_CAPSET_BMPCACHE2	19
 #define RDP_CAPLEN_BMPCACHE2	0x28
 #define BMPCACHE2_FLAG_PERSIST	((uint32)1<<31)
 
+#define RDP_CAPSET_MULTIFRAGMENTUPDATE 26
+#define RDP_CAPLEN_MULTIFRAGMENTUPDATE 8
+
+#define RDP_CAPSET_LARGE_POINTER	27
+#define RDP_CAPLEN_LARGE_POINTER	6
+
+#define RDP_CAPSET_VC	20
+#define RDP_CAPLEN_VC	0x08
+
 #define RDP_SOURCE		"MSTSC"
 
 /* Logon flags */
@@ -323,19 +410,20 @@ enum RDP_INPUT_DEVICE
 #define RDP_INFO_AUTOLOGON 	      0x00000008
 #define RDP_INFO_UNICODE              0x00000010
 #define RDP_INFO_MAXIMIZESHELL        0x00000020
-#define RDP_INFO_COMPRESSION	      0x00000080	/* mppc compression with 8kB histroy buffer */
+#define RDP_INFO_COMPRESSION	      0x00000080	/* mppc compression with 8kB history buffer */
 #define RDP_INFO_ENABLEWINDOWSKEY     0x00000100
 #define RDP_INFO_COMPRESSION2	      0x00000200	/* rdp5 mppc compression with 64kB history buffer */
 #define RDP_INFO_REMOTE_CONSOLE_AUDIO 0x00002000
 #define RDP_INFO_PASSWORD_IS_SC_PIN   0x00040000
 
-#define RDP5_DISABLE_NOTHING	0x00
-#define RDP5_NO_WALLPAPER	0x01
-#define RDP5_NO_FULLWINDOWDRAG	0x02
-#define RDP5_NO_MENUANIMATIONS	0x04
-#define RDP5_NO_THEMING		0x08
-#define RDP5_NO_CURSOR_SHADOW	0x20
-#define RDP5_NO_CURSORSETTINGS	0x40	/* disables cursor blinking */
+/* TS_EXTENDED_INFO_PACKET.performanceFlags */
+#define PERF_DISABLE_WALLPAPER	        0x01
+#define PERF_DISABLE_FULLWINDOWDRAG	0x02
+#define PERF_DISABLE_MENUANIMATIONS	0x04
+#define PERF_DISABLE_THEMING		0x08
+#define PERF_DISABLE_CURSOR_SHADOW	0x20
+#define PERF_DISABLE_CURSORSETTINGS	0x40	/* disables cursor blinking */
+#define PERF_ENABLE_FONT_SMOOTHING	0x80
 
 /* compression types */
 #define RDP_MPPC_BIG		0x01
@@ -436,6 +524,7 @@ enum RDP_INPUT_DEVICE
 #define RD_STATUS_NO_SUCH_FILE             0xc000000f
 #define RD_STATUS_INVALID_DEVICE_REQUEST   0xc0000010
 #define RD_STATUS_ACCESS_DENIED            0xc0000022
+#define RD_STATUS_BUFFER_TOO_SMALL         0xc0000023
 #define RD_STATUS_OBJECT_NAME_COLLISION    0xc0000035
 #define RD_STATUS_DISK_FULL                0xc000007f
 #define RD_STATUS_FILE_IS_A_DIRECTORY      0xc00000ba
@@ -481,31 +570,93 @@ enum RDP_INPUT_DEVICE
 #define FILE_DELETE_ON_CLOSE            0x00001000
 #define FILE_OPEN_FOR_FREE_SPACE_QUERY  0x00800000
 
-/* RDP5 disconnect PDU */
-#define exDiscReasonNoInfo				0x0000
-#define exDiscReasonAPIInitiatedDisconnect		0x0001
-#define exDiscReasonAPIInitiatedLogoff			0x0002
-#define exDiscReasonServerIdleTimeout			0x0003
-#define exDiscReasonServerLogonTimeout			0x0004
-#define exDiscReasonReplacedByOtherConnection		0x0005
-#define exDiscReasonOutOfMemory				0x0006
-#define exDiscReasonServerDeniedConnection		0x0007
-#define exDiscReasonServerDeniedConnectionFips		0x0008
-#define exDiscReasonServerInsufficientPrivileges        0x0009
-#define exDiscReasonServerFreshCredentialsRequired      0x000a
-#define exDiscReasonRPCInitiatedDisconnectByUser        0x000b
-#define exDiscReasonByUser                              0x000c
-#define exDiscReasonLicenseInternal			0x0100
-#define exDiscReasonLicenseNoLicenseServer		0x0101
-#define exDiscReasonLicenseNoLicense			0x0102
-#define exDiscReasonLicenseErrClientMsg			0x0103
-#define exDiscReasonLicenseHwidDoesntMatchLicense	0x0104
-#define exDiscReasonLicenseErrClientLicense		0x0105
-#define exDiscReasonLicenseCantFinishProtocol		0x0106
-#define exDiscReasonLicenseClientEndedProtocol		0x0107
-#define exDiscReasonLicenseErrClientEncryption		0x0108
-#define exDiscReasonLicenseCantUpgradeLicense		0x0109
-#define exDiscReasonLicenseNoRemoteConnections		0x010a
+#define CAP_GENERAL_TYPE   0x0001
+#define CAP_PRINTER_TYPE   0x0002
+#define CAP_PORT_TYPE      0x0003
+#define CAP_DRIVE_TYPE     0x0004
+#define CAP_SMARTCARD_TYPE 0x0005
+
+#define GENERAL_CAPABILITY_VERSION_01   0x00000001
+#define GENERAL_CAPABILITY_VERSION_02   0x00000002
+#define PRINT_CAPABILITY_VERSION_01     0x00000001
+#define PORT_CAPABILITY_VERSION_01      0x00000001
+#define DRIVE_CAPABILITY_VERSION_01     0x00000001
+#define DRIVE_CAPABILITY_VERSION_02     0x00000002
+#define SMARTCARD_CAPABILITY_VERSION_01 0x00000001
+
+#define RDPDR_IRP_MJ_CREATE                   0x00000001
+#define RDPDR_IRP_MJ_CLEANUP                  0x00000002
+#define RDPDR_IRP_MJ_CLOSE                    0x00000004
+#define RDPDR_IRP_MJ_READ                     0x00000008
+#define RDPDR_IRP_MJ_WRITE                    0x00000010
+#define RDPDR_IRP_MJ_FLUSH_BUFFERS            0x00000020
+#define RDPDR_IRP_MJ_SHUTDOWN                 0x00000040
+#define RDPDR_IRP_MJ_DEVICE_CONTROL           0x00000080
+#define RDPDR_IRP_MJ_QUERY_VOLUME_INFORMATION 0x00000100
+#define RDPDR_IRP_MJ_SET_VOLUME_INFORMATION   0x00000200
+#define RDPDR_IRP_MJ_QUERY_INFORMATION        0x00000400
+#define RDPDR_IRP_MJ_SET_INFORMATION          0x00000800
+#define RDPDR_IRP_MJ_DIRECTORY_CONTROL        0x00001000
+#define RDPDR_IRP_MJ_LOCK_CONTROL             0x00002000
+#define RDPDR_IRP_MJ_QUERY_SECURITY           0x00004000
+#define RDPDR_IRP_MJ_SET_SECURITY             0x00008000
+#define ALL_RDPDR_IRP_MJ                      0x0000FFFF
+
+#define RDPDR_PRINTER_ANNOUNCE_FLAG_ASCII		0x00000001
+#define RDPDR_PRINTER_ANNOUNCE_FLAG_DEFAULTPRINTER	0x00000002
+#define RDPDR_PRINTER_ANNOUNCE_FLAG_NETWORKPRINTER	0x00000004
+#define RDPDR_PRINTER_ANNOUNCE_FLAG_TSPRINTER		0x00000008
+#define RDPDR_PRINTER_ANNOUNCE_FLAG_XPSFORMAT		0x00000010
+
+#define RDPDR_DEVICE_REMOVE_PDUS      0x00000001
+#define RDPDR_CLIENT_DISPLAY_NAME_PDU 0x00000002
+#define RDPDR_USER_LOGGEDON_PDU       0x00000004
+
+/* RDP5 disconnect PDU
+ *
+ * Named after the corresponding names on the server side:
+ * https://msdn.microsoft.com/en-us/library/cc240544.aspx
+ */
+#define ERRINFO_UNSET                                   (unsigned)(-1)
+#define ERRINFO_NO_INFO					0x0000
+#define ERRINFO_RPC_INITIATED_DISCONNECT		0x0001
+#define ERRINFO_RPC_INITIATED_LOGOFF			0x0002
+#define ERRINFO_IDLE_TIMEOUT				0x0003
+#define ERRINFO_LOGON_TIMEOUT				0x0004
+#define ERRINFO_DISCONNECTED_BY_OTHERCONNECTION		0x0005
+#define ERRINFO_OUT_OF_MEMORY				0x0006
+#define ERRINFO_SERVER_DENIED_CONNECTION		0x0007
+#define ERRINFO_SERVER_DENIED_CONNECTION_FIPS		0x0008
+#define ERRINFO_SERVER_INSUFFICIENT_PRIVILEGES		0x0009
+#define ERRINFO_SERVER_FRESH_CREDENTIALS_REQUIRED	0x000a
+#define ERRINFO_RPC_INITIATED_DISCONNECT_BYUSER		0x000b
+#define ERRINFO_LOGOFF_BYUSER				0x000c
+#define ERRINFO_LICENSE_INTERNAL			0x0100
+#define ERRINFO_LICENSE_NO_LICENSE_SERVER		0x0101
+#define ERRINFO_LICENSE_NO_LICENSE			0x0102
+#define ERRINFO_LICENSE_BAD_CLIENT_MSG			0x0103
+#define ERRINFO_LICENSE_HWID_DOESNT_MATCH_LICENSE	0x0104
+#define ERRINFO_LICENSE_BAD_CLIENT_LICENSE		0x0105
+#define ERRINFO_LICENSE_CANT_FINISH_PROTOCOL		0x0106
+#define ERRINFO_LICENSE_CLIENT_ENDED_PROTOCOL		0x0107
+#define ERRINFO_LICENSE_BAD_CLIENT_ENCRYPTION		0x0108
+#define ERRINFO_LICENSE_CANT_UPGRADE_LICENSE		0x0109
+#define ERRINFO_LICENSE_NO_REMOTE_CONNECTIONS		0x010a
+#define ERRINFO_CB_DESTINATION_NOT_FOUND		0x0400
+#define ERRINFO_CB_LOADING_DESTINATION			0x0402
+#define ERRINFO_CB_REDIRECTING_TO_DESTINATION		0x0404
+#define ERRINFO_CB_SESSION_ONLINE_VM_WAKE		0x0405
+#define ERRINFO_CB_SESSION_ONLINE_VM_BOOT		0x0406
+#define ERRINFO_CB_SESSION_ONLINE_VM_NO_DNS		0x0407
+#define ERRINFO_CB_DESTINATION_POOL_NOT_FREE		0x0408
+#define ERRINFO_CB_CONNECTION_CANCELLED			0x0409
+#define ERRINFO_CB_CONNECTION_ERROR_INVALID_SETTINGS	0x0410
+#define ERRINFO_CB_SESSION_ONLINE_VM_BOOT_TIMEOUT	0x0411
+#define ERRINFO_CB_SESSION_ONLINE_VM_SESSMON_FAILED	0x0412
+#define ERRINFO_REMOTEAPPSNOTENABLED			0x10f3
+#define ERRINFO_UPDATESESSIONKEYFAILED			0x1191
+#define ERRINFO_DECRYPTFAILED				0x1192
+#define ERRINFO_ENCRYPTFAILED				0x1193
 
 /* SeamlessRDP constants */
 #define SEAMLESSRDP_NOTYETMAPPED -1
@@ -531,15 +682,144 @@ enum RDP_INPUT_DEVICE
 /* redirect flags, from [MS-RDPBCGR] 2.2.13.1 */
 enum RDP_PDU_REDIRECT_FLAGS
 {
-	PDU_REDIRECT_HAS_IP = 0x1,
-	PDU_REDIRECT_HAS_LOAD_BALANCE_INFO = 0x2,
-	PDU_REDIRECT_HAS_USERNAME = 0x4,
-	PDU_REDIRECT_HAS_DOMAIN = 0x8,
-	PDU_REDIRECT_HAS_PASSWORD = 0x10,
-	PDU_REDIRECT_DONT_STORE_USERNAME = 0x20,
-	PDU_REDIRECT_USE_SMARTCARD = 0x40,
-	PDU_REDIRECT_INFORMATIONAL = 0x80,
-	PDU_REDIRECT_HAS_TARGET_FQDN = 0x100,
-	PDU_REDIRECT_HAS_TARGET_NETBIOS = 0x200,
-	PDU_REDIRECT_HAS_TARGET_IP_ARRAY = 0x800
+	LB_TARGET_NET_ADDRESS = 0x1,
+	LB_LOAD_BALANCE_INFO = 0x2,
+	LB_USERNAME = 0x4,
+	LB_DOMAIN = 0x8,
+	LB_PASSWORD = 0x10,
+	LB_DONTSTOREUSERNAME = 0x20,
+	LB_SMARTCARD_LOGON = 0x40,
+	LB_NOREDIRECT = 0x80,
+	LB_TARGET_FQDN = 0x100,
+	LB_TARGET_NETBIOS = 0x200,
+	LB_TARGET_NET_ADDRESSES = 0x800,
+	LB_CLIENT_TSV_URL = 0x1000,
+	LB_SERVER_TSV_CAPABLE = 0x2000,
+	LB_PASSWORD_IS_PK_ENCRYPTED = 0x4000,
+	LB_REDIRECTION_GUID = 0x8000,
+	LB_TARGET_CERTIFICATE = 0x10000
 };
+
+/* desktop orientation */
+enum RDP_DESKTOP_ORIENTATION
+{
+	ORIENTATION_LANDSCAPE = 0,
+	ORIENTATION_PORTRAIT = 90,
+	ORIENTATION_LANDSCAPE_FLIPPED = 180,
+	ORIENTATION_PORTRAIT_FLIPPED = 270
+};
+/* color depths, from [MS-RDPBCGR] 2.2.1.3.2 */
+#define RNS_UD_COLOR_4BPP	0xCA00
+#define RNS_UD_COLOR_8BPP	0xCA01
+#define RNS_UD_COLOR_16BPP_555	0xCA02
+#define RNS_UD_COLOR_16BPP_565	0xCA03
+#define RNS_UD_COLOR_24BPP	0xCA04
+
+#define RNS_UD_SAS_DEL		0xAA03
+
+/* version, [MS-RDPBCGR] 2.2.1.3.2 */
+#define RDP_40		0x00080001	/* RDP 4.0 clients */
+#define RDP_50		0x00080004	/* RDP 5.0, 5.1, 5.2, 6.0, 6.1, 7.0, 7.1, 8.0, and 8.1 clients */
+#define RDP_10_0	0x00080005	/* RDP 10.0 clients */
+#define RDP_10_1	0x00080006	/* RDP 10.1 clients */
+#define RDP_10_2	0x00080007	/* RDP 10.2 clients */
+#define RDP_10_3	0x00080008	/* RDP 10.3 clients */
+
+/* supportedColorDepths, [MS-RDPBCGR] 2.2.1.3.2 */
+#define RNS_UD_24BPP_SUPPORT	0x0001
+#define RNS_UD_16BPP_SUPPORT	0x0002
+#define RNS_UD_15BPP_SUPPORT	0x0004
+#define RNS_UD_32BPP_SUPPORT	0x0008
+
+/* earlyCapabilityFlags, [MS-RDPBCGR] 2.2.1.3.2 */
+#define RNS_UD_CS_SUPPORT_ERRINFO_PDU		0x0001
+#define RNS_UD_CS_WANT_32BPP_SESSION		0x0002
+#define RNS_UD_CS_SUPPORT_STATUSINFO_PDU	0x0004
+#define RNS_UD_CS_STRONG_ASYMMETRIC_KEYS	0x0008
+#define RNS_UD_CS_UNUSED			0x0010
+#define RNS_UD_CS_VALID_CONNECTION_TYPE		0x0020
+#define RNS_UD_CS_SUPPORT_MONITOR_LAYOUT_PDU	0x0040
+#define RNS_UD_CS_SUPPORT_NETCHAR_AUTODETECT	0x0080
+#define RNS_UD_CS_SUPPORT_DYNVC_GFX_PROTOCOL	0x0100
+#define RNS_UD_CS_SUPPORT_DYNAMIC_TIME_ZONE	0x0200
+#define RNS_UD_CS_SUPPORT_HEARTBEAT_PDU		0x0400
+
+/* [MS-RDPBCGR] 2.2.7.1.1 */
+#define OSMAJORTYPE_WINDOWS	0x0001
+#define OSMINORTYPE_WINDOWSNT	0x0003
+#define TS_CAPS_PROTOCOLVERSION	0x0200
+
+/* extraFlags, [MS-RDPBCGR] 2.2.7.1.1 */
+#define FASTPATH_OUTPUT_SUPPORTED	0x0001
+#define LONG_CREDENTIALS_SUPPORTED	0x0004
+#define AUTORECONNECT_SUPPORTED		0x0008
+#define ENC_SALTED_CHECKSUM		0x0010
+#define NO_BITMAP_COMPRESSION_HDR	0x0400
+
+/* [MS-RDPBCGR], TS_BITMAP_DATA, flags */
+#define BITMAP_COMPRESSION              0x0001
+
+/* orderFlags, [MS-RDPBCGR] 2.2.7.1.3 */
+#define NEGOTIATEORDERSUPPORT	0x0002
+#define ZEROBOUNDSDELTASSUPPORT 0x0008
+#define COLORINDEXSUPPORT	0x0020
+#define SOLIDPATTERNBRUSHONLY	0x0040
+#define ORDERFLAGS_EXTRA_FLAGS	0x0080
+
+/* orderSupport index, [MS-RDPBCGR] 2.2.7.1.3 */
+#define TS_NEG_DSTBLT_INDEX		0x00
+#define TS_NEG_PATBLT_INDEX		0x01
+#define TS_NEG_SCRBLT_INDEX		0x02
+#define TS_NEG_MEMBLT_INDEX		0x03
+#define TS_NEG_MEM3BLT_INDEX		0x04
+#define TS_NEG_DRAWNINEGRID_INDEX	0x07
+#define TS_NEG_LINETO_INDEX		0x08
+#define TS_NEG_MULTI_DRAWNINEGRID_INDEX 0x09
+#define TS_NEG_SAVEBITMAP_INDEX		0x0B
+#define TS_NEG_MULTIDSTBLT_INDEX	0x0F
+#define TS_NEG_MULTIPATBLT_INDEX	0x10
+#define TS_NEG_MULTISCRBLT_INDEX	0x11
+#define TS_NEG_MULTIOPAQUERECT_INDEX	0x12
+#define TS_NEG_FAST_INDEX_INDEX		0x13
+#define TS_NEG_POLYGON_SC_INDEX		0x14
+#define TS_NEG_POLYGON_CB_INDEX		0x15
+#define TS_NEG_POLYLINE_INDEX		0x16
+#define TS_NEG_FAST_GLYPH_INDEX		0x18
+#define TS_NEG_ELLIPSE_SC_INDEX		0x19
+#define TS_NEG_ELLIPSE_CB_INDEX		0x1A
+#define TS_NEG_INDEX_INDEX		0x1B
+
+/* [MS-RDPBCGR] 2.2.7.1.6 */
+#define INPUT_FLAG_SCANCODES		0x0001
+#define INPUT_FLAG_MOUSEX		0x0004
+#define INPUT_FLAG_FASTPATH_INPUT	0x0008
+#define INPUT_FLAG_UNICODE		0x0010
+#define INPUT_FLAG_FASTPATH_INPUT2	0x0020
+#define INPUT_FLAG_UNUSED1		0x0040
+#define INPUT_FLAG_UNUSED2		0x0080
+#define TS_INPUT_FLAG_MOUSE_HWHEEL	0x0100
+#define TS_INPUT_FLAG_QOE_TIMESTAMPS	0x0200
+
+/* [MS-RDPBCGR] 2.2.7.1.8 */
+#define GLYPH_SUPPORT_NONE    0x0000
+#define GLYPH_SUPPORT_PARTIAL 0x0001
+#define GLYPH_SUPPORT_FULL    0x0002
+#define GLYPH_SUPPORT_ENCODE  0x0003
+
+/* [MS-RDPBCGR] 2.2.7.1.11 */
+#define SOUND_BEEPS_FLAG 0x0001
+
+/* [MS-RDPBCGR] 2.2.7.2.5 */
+#define FONTSUPPORT_FONTLIST 0x0001
+
+/* [MS-RDPBCGR] 2.2.7.2.7 */
+#define LARGE_POINTER_FLAG_96x96	1
+
+/* [MS-RDPBCGR] TS_SUPPRESS_OUTPUT_PDU allowDisplayUpdates */
+enum RDP_SUPPRESS_STATUS
+{
+	SUPPRESS_DISPLAY_UPDATES = 0x00,
+	ALLOW_DISPLAY_UPDATES = 0x01
+};
+
+#endif /* _CONSTANTS_H */
