@@ -44,13 +44,14 @@ static void lspci_send(const char *output);
 static RD_BOOL
 handle_child_line(const char *line, void *data)
 {
+	UNUSED(data);
 	const char *val;
 	char buf[1024];
 
 	if (str_startswith(line, "Class:"))
 	{
 		val = line + sizeof("Class:");
-		/* Skip whitespace and second Class: occurance */
+		/* Skip whitespace and second Class: occurrence */
 		val += strspn(val, " \t") + sizeof("Class");
 		current_device.klass = strtol(val, NULL, 16);
 	}
@@ -99,7 +100,7 @@ handle_child_line(const char *line, void *data)
 	}
 	else
 	{
-		warning("lspci: Unrecoqnized line '%s'\n", line);
+		logger(Core, Warning, "handle_child_line(), Unrecognized lspci line '%s'", line);
 	}
 	return True;
 }
@@ -109,6 +110,7 @@ handle_child_line(const char *line, void *data)
 static RD_BOOL
 lspci_process_line(const char *line, void *data)
 {
+	UNUSED(data);
 	char *lspci_command[5] = { "lspci", "-m", "-n", "-v", NULL };
 
 	if (!strcmp(line, "LSPCI"))
@@ -120,7 +122,7 @@ lspci_process_line(const char *line, void *data)
 	}
 	else
 	{
-		error("lspci protocol error: Invalid line '%s'\n", line);
+		logger(Core, Error, "lspci_process_line(), invalid line '%s'", line);
 	}
 	return True;
 }
@@ -139,11 +141,6 @@ lspci_process(STREAM s)
 	buf = xmalloc(pkglen + 1);
 	in_uint8a(s, buf, pkglen);
 	buf[pkglen] = '\0';
-#if 0
-	printf("lspci recv:\n");
-	hexdump(s->p, pkglen);
-#endif
-
 	str_handle_lines(buf, &rest, lspci_process_line, NULL);
 	xfree(buf);
 }
@@ -168,12 +165,6 @@ lspci_send(const char *output)
 	len = strlen(output);
 	s = channel_init(lspci_channel, len);
 	out_uint8a(s, output, len) s_mark_end(s);
-
-#if 0
-	printf("lspci send:\n");
-	hexdump(s->channel_hdr + 8, s->end - s->channel_hdr - 8);
-#endif
-
 	channel_send(s, lspci_channel);
 	s_free(s);
 }
