@@ -3,7 +3,7 @@
    Seamless Windows support
    Copyright 2005-2008 Peter Astrand <astrand@cendio.se> for Cendio AB
    Copyright 2007-2008 Pierre Ossman <ossman@cendio.se> for Cendio AB
-   Copyright 2013-2014 Henrik Andersson  <hean01@cendio.se> for Cendio AB   
+   Copyright 2013-2017 Henrik Andersson  <hean01@cendio.se> for Cendio AB
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -23,12 +23,6 @@
 #include <stdarg.h>
 #include <assert.h>
 
-#ifdef WITH_DEBUG_SEAMLESS
-#define DEBUG_SEAMLESS(args) printf args;
-#else
-#define DEBUG_SEAMLESS(args)
-#endif
-
 extern RD_BOOL g_seamless_rdp;
 static VCHANNEL *seamless_channel;
 static unsigned int seamless_serial;
@@ -62,18 +56,19 @@ seamless_get_token(char **s)
 static RD_BOOL
 seamless_process_line(const char *line, void *data)
 {
+	UNUSED(data);
 	char *p, *l;
-	char *tok1, *tok2, *tok3, *tok4, *tok5, *tok6, *tok7, *tok8;
+	char *tok1, *tok3, *tok4, *tok5, *tok6, *tok7, *tok8;
 	unsigned long id, flags;
 	char *endptr;
 
 	l = xstrdup(line);
 	p = l;
 
-	DEBUG_SEAMLESS(("seamlessrdp got:%s\n", p));
+	logger(Core, Debug, "seamless_process_line(), got '%s'", p);
 
 	tok1 = seamless_get_token(&p);
-	tok2 = seamless_get_token(&p);
+	(void) seamless_get_token(&p);
 	tok3 = seamless_get_token(&p);
 	tok4 = seamless_get_token(&p);
 	tok5 = seamless_get_token(&p);
@@ -176,7 +171,7 @@ seamless_process_line(const char *line, void *data)
 
 			if ((size_t)len >= sizeof(icon_buf))
 			{
-				warning("seamless_process_line(), icon data would overrun icon_buf");
+				logger(Protocol, Warning, "seamless_process_line(), icon data would overrun icon_buf");
 				break;
 			}
 		}
@@ -291,7 +286,7 @@ seamless_process_line(const char *line, void *data)
 	}
 	else if (!strcmp("DEBUG", tok1))
 	{
-		DEBUG_SEAMLESS(("SeamlessRDP:%s\n", line));
+		logger(Core, Debug, "seamless_process_line(), %s", line);
 	}
 	else if (!strcmp("SYNCBEGIN", tok1))
 	{
@@ -370,7 +365,7 @@ seamless_line_handler(const char *line, void *data)
 {
 	if (!seamless_process_line(line, data))
 	{
-		warning("SeamlessRDP: Invalid request:%s\n", line);
+		logger(Core, Warning, "seamless_line_handler(), invalid request '%s'", line);
 	}
 	return True;
 }
@@ -387,11 +382,6 @@ seamless_process(STREAM s)
 	buf = xmalloc(pkglen + 1);
 	in_uint8a(s, buf, pkglen);
 	buf[pkglen] = '\0';
-#if 0
-	printf("seamless recv:\n");
-	hexdump(buf, pkglen);
-#endif
-
 	str_handle_lines(buf, &seamless_rest, seamless_line_handler, NULL);
 
 	xfree(buf);
@@ -453,12 +443,7 @@ seamless_send(const char *command, const char *format, ...)
 	s = channel_init(seamless_channel, len);
 	out_uint8a(s, buf, len) s_mark_end(s);
 
-	DEBUG_SEAMLESS(("seamlessrdp sending:%s", buf));
-
-#if 0
-	printf("seamless send:\n");
-	hexdump(s->channel_hdr + 8, s->end - s->channel_hdr - 8);
-#endif
+	logger(Core, Debug, "seamless_send(), sending '%s'", buf);
 
 	channel_send(s, seamless_channel);
 	s_free(s);
@@ -557,8 +542,10 @@ seamless_send_persistent(RD_BOOL enable)
 	unsigned int res;
 	if (!g_seamless_rdp)
 		return (unsigned int) -1;
-	printf("%s persistent seamless mode.\n", enable?"Enable":"Disable");
+
+	logger(Core, Debug, "seamless_send_persistent(), %s persistent seamless mode",
+	       enable ? "enable" : "disable");
 	res = seamless_send("PERSISTENT", "%d", enable);
-	
+
 	return res;
 }
