@@ -3,6 +3,8 @@
    Secure sockets abstraction layer
    Copyright (C) Matthew Chapman 1999-2008
    Copyright (C) Jay Sorg 2006-2008
+   Copyright 2017 Henrik Andersson <hean01@cendio.se> for Cendio AB
+   Copyright 2017 Alexander Zakharov <uglym8@gmail.com>
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -21,24 +23,19 @@
 #ifndef _RDSSL_H
 #define _RDSSL_H
 
-#include <openssl/rc4.h>
-#include <openssl/md5.h>
-#include <openssl/sha.h>
-#include <openssl/bn.h>
-#include <openssl/x509v3.h>
-#include <openssl/hmac.h>
+#include <nettle/md5.h>
+#include <nettle/sha1.h>
+#include <nettle/arcfour.h>
+#include <nettle/hmac.h>
+#include <nettle/rsa.h>
 
-#if defined(OPENSSL_VERSION_NUMBER) && (OPENSSL_VERSION_NUMBER >= 0x0090800f)
-#define D2I_X509_CONST const
-#else
-#define D2I_X509_CONST
-#endif
+#include <gnutls/x509.h>
 
-#define RDSSL_RC4 RC4_KEY
-#define RDSSL_SHA1 SHA_CTX
-#define RDSSL_MD5 MD5_CTX
-#define RDSSL_CERT X509
-#define RDSSL_RKEY RSA
+#define RDSSL_RC4 struct arcfour_ctx
+#define RDSSL_SHA1 struct sha1_ctx
+#define RDSSL_MD5 struct md5_ctx
+#define RDSSL_CERT gnutls_x509_crt_t
+#define RDSSL_RKEY struct rsa_public_key
 
 void rdssl_sha1_init(RDSSL_SHA1 * sha1);
 void rdssl_sha1_update(RDSSL_SHA1 * sha1, uint8 * data, uint32 len);
@@ -63,5 +60,4 @@ RD_BOOL rdssl_sig_ok(uint8 * exponent, uint32 exp_len, uint8 * modulus, uint32 m
 
 void rdssl_hmac_md5(const void *key, int key_len,
 		    const unsigned char *msg, int msg_len, unsigned char *md);
-
 #endif
