@@ -4,6 +4,8 @@
    Copyright 2003-2008 Erik Forsberg <forsberg@cendio.se> for Cendio AB
    Copyright (C) Matthew Chapman <matthewc.unsw.edu.au> 2003-2008
    Copyright 2006-2011 Pierre Ossman <ossman@cendio.se> for Cendio AB
+   Copyright 2016 Alexander Zakharov <uglym8@gmail.com>
+   Copyright 2017 Henrik Andersson <hean01@cendio.se> for Cendio AB
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -32,15 +34,11 @@
     http://msdn.microsoft.com/library/en-us/winui/winui/windowsuserinterface/dataexchange/clipboard/clipboardformats.asp
 */
 
-#ifdef HAVE_ICONV
 #ifdef HAVE_LANGINFO_H
-#ifdef HAVE_ICONV_H
 #include <langinfo.h>
 #include <iconv.h>
 #define USE_UNICODE_CLIPBOARD
 #endif
-#endif
-#endif
 
 #ifdef USE_UNICODE_CLIPBOARD
 #define RDP_CF_TEXT CF_UNICODETEXT
@@ -157,7 +155,7 @@ utf16_lf2crlf(uint8 * data, uint32 * size)
 {
 	uint8 *result;
 	uint16 *inptr, *outptr;
-	RD_BOOL swap_endianess;
+	RD_BOOL swap_endianness;
 
 	/* Worst case: Every char is LF */
 	result = xmalloc((*size * 2) + 2);
@@ -168,16 +166,16 @@ utf16_lf2crlf(uint8 * data, uint32 * size)
 	outptr = (uint16 *) result;
 
 	/* Check for a reversed BOM */
-	swap_endianess = (*inptr == 0xfffe);
+	swap_endianness = (*inptr == 0xfffe);
 
 	uint16 uvalue_previous = 0;	/* Kept so we'll avoid translating CR-LF to CR-CR-LF */
 	while ((uint8 *) inptr < data + *size)
 	{
 		uint16 uvalue = *inptr;
-		if (swap_endianess)
+		if (swap_endianness)
 			uvalue = ((uvalue << 8) & 0xff00) + (uvalue >> 8);
 		if ((uvalue == 0x0a) && (uvalue_previous != 0x0d))
-			*outptr++ = swap_endianess ? 0x0d00 : 0x0d;
+			*outptr++ = swap_endianness ? 0x0d00 : 0x0d;
 		uvalue_previous = uvalue;
 		*outptr++ = *inptr++;
 	}
@@ -222,8 +220,15 @@ xclip_provide_selection(XSelectionRequestEvent * req, Atom type, unsigned int fo
 			uint32 length)
 {
 	XEvent xev;
+	char *target_name, *property_name;
 
-	DEBUG_CLIPBOARD(("xclip_provide_selection: requestor=0x%08x, target=%s, property=%s, length=%u\n", (unsigned) req->requestor, XGetAtomName(g_display, req->target), XGetAtomName(g_display, req->property), (unsigned) length));
+	target_name = XGetAtomName(g_display, req->target);
+	property_name = XGetAtomName(g_display, req->property);
+	logger(Clipboard, Debug,
+	       "xclip_provide_selection(), requestor=0x%08x, target=%s, property=%s, length=%u",
+	       (unsigned) req->requestor, target_name, property_name, (unsigned) length);
+	XFree(target_name);
+	XFree(property_name);
 
 	XChangeProperty(g_display, req->requestor, req->property,
 			type, format, PropModeReplace, data, length);
@@ -246,10 +251,15 @@ static void
 xclip_refuse_selection(XSelectionRequestEvent * req)
 {
 	XEvent xev;
+	char *target_name, *property_name;
 
-	DEBUG_CLIPBOARD(("xclip_refuse_selection: requestor=0x%08x, target=%s, property=%s\n",
-			 (unsigned) req->requestor, XGetAtomName(g_display, req->target),
-			 XGetAtomName(g_display, req->property)));
+	target_name = XGetAtomName(g_display, req->target);
+	property_name = XGetAtomName(g_display, req->property);
+	logger(Clipboard, Debug,
+	       "xclip_refuse_selection(), requestor=0x%08x, target=%s, property=%s",
+	       (unsigned) req->requestor, target_name, property_name);
+	XFree(target_name);
+	XFree(property_name);
 
 	xev.xselection.type = SelectionNotify;
 	xev.xselection.serial = 0;
@@ -290,8 +300,12 @@ helper_cliprdr_send_empty_response()
 static RD_BOOL
 xclip_send_data_with_convert(uint8 * source, size_t source_size, Atom target)
 {
-	DEBUG_CLIPBOARD(("xclip_send_data_with_convert: target=%s, size=%u\n",
-			 XGetAtomName(g_display, target), (unsigned) source_size));
+	char *target_name;
+
+	target_name = XGetAtomName(g_display, target);
+	logger(Clipboard, Debug, "xclip_send_data_with_convert(), target=%s, size=%u",
+	       target_name, (unsigned) source_size);
+	XFree(target_name);
 
 #ifdef USE_UNICODE_CLIPBOARD
 	if (target == format_string_atom ||
@@ -321,7 +335,9 @@ xclip_send_data_with_convert(uint8 * source, size_t source_size, Atom target)
 			cd = iconv_open(WINDOWS_CODEPAGE, locale_charset);
 			if (cd == (iconv_t) - 1)
 			{
-				DEBUG_CLIPBOARD(("Locale charset %s not found in iconv. Unable to convert clipboard text.\n", locale_charset));
+				logger(Clipboard, Error,
+				       "xclip_send_data_with_convert(), convert failed, locale charset %s not found",
+				       locale_charset);
 				return False;
 			}
 			unicode_buffer_size = source_size * 4;
@@ -357,7 +373,7 @@ xclip_send_data_with_convert(uint8 * source, size_t source_size, Atom target)
 		unicode_buffer_remaining = unicode_buffer;
 		data_remaining = (char *) source;
 		data_size_remaining = source_size;
-		iconv(cd, (ICONV_CONST char **) &data_remaining, &data_size_remaining,
+		iconv(cd, (char **) &data_remaining, &data_size_remaining,
 		      &unicode_buffer_remaining, &unicode_buffer_size_remaining);
 		iconv_close(cd);
 
@@ -366,8 +382,9 @@ xclip_send_data_with_convert(uint8 * source, size_t source_size, Atom target)
 		translated_data = utf16_lf2crlf((uint8 *) unicode_buffer, &translated_data_size);
 		if (translated_data != NULL)
 		{
-			DEBUG_CLIPBOARD(("Sending Unicode string of %d bytes\n",
-					 translated_data_size));
+			logger(Clipboard, Debug,
+			       "xclip_send_data_with_convert(), sending unicode string of %d bytes",
+			       translated_data_size);
 			helper_cliprdr_send_response(translated_data, translated_data_size);
 			xfree(translated_data);	/* Not the same thing as XFree! */
 		}
@@ -385,11 +402,12 @@ xclip_send_data_with_convert(uint8 * source, size_t source_size, Atom target)
 		if (rdp_clipboard_request_format != RDP_CF_TEXT)
 			return False;
 
-		DEBUG_CLIPBOARD(("Translating linebreaks before sending data\n"));
+		logger(Clipboard, Debug,
+		       "xclip_send_data_with_convert(), translating linebreaks before sending data");
 		translated_data = lf2crlf(source, &length);
 		if (translated_data != NULL)
 		{
-			helper_cliprdr_send_response(translated_data, length+1);
+			helper_cliprdr_send_response(translated_data, length + 1);
 			xfree(translated_data);	/* Not the same thing as XFree! */
 		}
 
@@ -430,12 +448,13 @@ xclip_probe_selections()
 
 	if (probing_selections)
 	{
-		DEBUG_CLIPBOARD(("Already probing selections. Scheduling reprobe.\n"));
+		logger(Clipboard, Debug,
+		       "xclip_probe_selection(), already probing selections, scheduling reprobe");
 		reprobe_selections = True;
 		return;
 	}
 
-	DEBUG_CLIPBOARD(("Probing selections.\n"));
+	logger(Clipboard, Debug, "xclip_probe_selection(), probing selections");
 
 	probing_selections = True;
 	reprobe_selections = False;
@@ -481,7 +500,7 @@ xclip_probe_selections()
 		return;
 	}
 
-	DEBUG_CLIPBOARD(("No owner of any selection.\n"));
+	logger(Clipboard, Debug, "xclip_probe_selection(), no owner of any selection");
 
 	/* FIXME:
 	   Without XFIXES, we cannot reliably know the formats offered by an
@@ -497,24 +516,29 @@ xclip_probe_selections()
    The SelectionNotify event is sent from the clipboard owner to the requestor
    after his request was satisfied.
    If this function is called, we're the requestor side. */
-#ifndef MAKE_PROTO
 void
 xclip_handle_SelectionNotify(XSelectionEvent * event)
 {
-	unsigned long nitems, bytes_left;
+	unsigned long i, nitems, bytes_left;
 	XWindowAttributes wa;
 	Atom type;
 	Atom *supported_targets;
-	int res, i, format;
+	int res, format;
 	uint8 *data = NULL;
+	char *selection_name, *target_name, *property_name;
 
 	if (event->property == None)
 		goto fail;
 
-	DEBUG_CLIPBOARD(("xclip_handle_SelectionNotify: selection=%s, target=%s, property=%s\n",
-			 XGetAtomName(g_display, event->selection),
-			 XGetAtomName(g_display, event->target),
-			 XGetAtomName(g_display, event->property)));
+	selection_name = XGetAtomName(g_display, event->selection);
+	target_name = XGetAtomName(g_display, event->target);
+	property_name = XGetAtomName(g_display, event->property);
+	logger(Clipboard, Debug,
+	       "xclip_handle_SelectionNotify(), selection=%s, target=%s, property=%s",
+	       selection_name, target_name, property_name);
+	XFree(selection_name);
+	XFree(target_name);
+	XFree(property_name);
 
 	if (event->target == timestamp_atom)
 	{
@@ -536,7 +560,8 @@ xclip_handle_SelectionNotify(XSelectionEvent * event)
 
 		if ((res != Success) || (nitems != 1) || (format != 32))
 		{
-			DEBUG_CLIPBOARD(("XGetWindowProperty failed!\n"));
+			logger(Clipboard, Error,
+			       "xclip_handle_SelectionNotify(), XGetWindowProperty failed");
 			goto fail;
 		}
 
@@ -546,8 +571,9 @@ xclip_handle_SelectionNotify(XSelectionEvent * event)
 			if (primary_timestamp == 0)
 				primary_timestamp++;
 			XDeleteProperty(g_display, g_wnd, rdesktop_primary_timestamp_target_atom);
-			DEBUG_CLIPBOARD(("Got PRIMARY timestamp: %u\n",
-					 (unsigned) primary_timestamp));
+			logger(Clipboard, Debug,
+			       "xclip_handle_SelectionNotify(), got PRIMARY timestamp: %u",
+			       (unsigned) primary_timestamp);
 		}
 		else
 		{
@@ -555,8 +581,9 @@ xclip_handle_SelectionNotify(XSelectionEvent * event)
 			if (clipboard_timestamp == 0)
 				clipboard_timestamp++;
 			XDeleteProperty(g_display, g_wnd, rdesktop_clipboard_timestamp_target_atom);
-			DEBUG_CLIPBOARD(("Got CLIPBOARD timestamp: %u\n",
-					 (unsigned) clipboard_timestamp));
+			logger(Clipboard, Debug,
+			       "xclip_handle_SelectionNotify(), got CLIPBOARD timestamp: %u",
+			       (unsigned) clipboard_timestamp);
 		}
 
 		XFree(data);
@@ -565,14 +592,16 @@ xclip_handle_SelectionNotify(XSelectionEvent * event)
 		{
 			if (primary_timestamp > clipboard_timestamp)
 			{
-				DEBUG_CLIPBOARD(("PRIMARY is most recent selection.\n"));
+				logger(Clipboard, Debug,
+				       "xclip_handle_SelectionNotify(), PRIMARY is most recent selection");
 				XConvertSelection(g_display, primary_atom, targets_atom,
 						  rdesktop_clipboard_target_atom, g_wnd,
 						  event->time);
 			}
 			else
 			{
-				DEBUG_CLIPBOARD(("CLIPBOARD is most recent selection.\n"));
+				logger(Clipboard, Debug,
+				       "xclip_handle_SelectionNotify(), CLIPBOARD is most recent selection");
 				XConvertSelection(g_display, clipboard_atom, targets_atom,
 						  rdesktop_clipboard_target_atom, g_wnd,
 						  event->time);
@@ -597,13 +626,14 @@ xclip_handle_SelectionNotify(XSelectionEvent * event)
 
 	if (res != Success)
 	{
-		DEBUG_CLIPBOARD(("XGetWindowProperty failed!\n"));
+		logger(Clipboard, Error,
+		       "xclip_handle_SelectionNotify(), XGetWindowProperty() failed");
 		goto fail;
 	}
 
 	if (type == incr_atom)
 	{
-		DEBUG_CLIPBOARD(("Received INCR.\n"));
+		logger(Clipboard, Debug, "xclip_handle_SelectionNotify(), received INCR");
 
 		XGetWindowAttributes(g_display, g_wnd, &wa);
 		if ((wa.your_event_mask | PropertyChangeMask) != wa.your_event_mask)
@@ -611,6 +641,7 @@ xclip_handle_SelectionNotify(XSelectionEvent * event)
 			XSelectInput(g_display, g_wnd, (wa.your_event_mask | PropertyChangeMask));
 		}
 		XFree(data);
+		data = NULL;
 		g_incr_target = event->target;
 		g_waiting_for_INCR = 1;
 		goto end;
@@ -630,13 +661,17 @@ xclip_handle_SelectionNotify(XSelectionEvent * event)
 			supported_targets = (Atom *) data;
 			for (i = 0; i < nitems; i++)
 			{
-				DEBUG_CLIPBOARD(("Target %d: %s\n", i,
-						 XGetAtomName(g_display, supported_targets[i])));
+				target_name = XGetAtomName(g_display, supported_targets[i]);
+				logger(Clipboard, Debug,
+				       "xclip_handle_SelectionNotify(), target %d: %s", i,
+				       target_name);
+				XFree(target_name);
 				if (supported_targets[i] == format_string_atom)
 				{
 					if (text_target_satisfaction < 1)
 					{
-						DEBUG_CLIPBOARD(("Other party supports STRING, choosing that as best_target\n"));
+						logger(Clipboard, Debug,
+						       "xclip_handle_SelectionNotify(), other party supports STRING, choosing that as best_target");
 						best_text_target = supported_targets[i];
 						text_target_satisfaction = 1;
 					}
@@ -646,7 +681,8 @@ xclip_handle_SelectionNotify(XSelectionEvent * event)
 				{
 					if (text_target_satisfaction < 2)
 					{
-						DEBUG_CLIPBOARD(("Other party supports text/unicode, choosing that as best_target\n"));
+						logger(Clipboard, Debug,
+						       "xclip_handle_SelectionNotify(), other party supports text/unicode, choosing that as best_target");
 						best_text_target = supported_targets[i];
 						text_target_satisfaction = 2;
 					}
@@ -655,7 +691,8 @@ xclip_handle_SelectionNotify(XSelectionEvent * event)
 				{
 					if (text_target_satisfaction < 3)
 					{
-						DEBUG_CLIPBOARD(("Other party supports UTF8_STRING, choosing that as best_target\n"));
+						logger(Clipboard, Debug,
+						       "xclip_handle_SelectionNotify(), other party supports UTF8_STRING, choosing that as best_target");
 						best_text_target = supported_targets[i];
 						text_target_satisfaction = 3;
 					}
@@ -665,7 +702,8 @@ xclip_handle_SelectionNotify(XSelectionEvent * event)
 				{
 					if (probing_selections && (text_target_satisfaction < 4))
 					{
-						DEBUG_CLIPBOARD(("Other party supports native formats, choosing that as best_target\n"));
+						logger(Clipboard, Debug,
+						       "xclip_handle_SelectionNotify(), other party supports native formats, choosing that as best_target");
 						best_text_target = supported_targets[i];
 						text_target_satisfaction = 4;
 					}
@@ -686,7 +724,8 @@ xclip_handle_SelectionNotify(XSelectionEvent * event)
 		}
 		else
 		{
-			DEBUG_CLIPBOARD(("Unable to find a textual target to satisfy RDP clipboard text request\n"));
+			logger(Clipboard, Error,
+			       "xclip_handle_SelectionNotify(), unable to find a textual target to satisfy RDP clipboard text request");
 			goto fail;
 		}
 	}
@@ -712,7 +751,8 @@ xclip_handle_SelectionNotify(XSelectionEvent * event)
 			if (primary_owner != clipboard_owner)
 				goto fail;
 
-			DEBUG_CLIPBOARD(("Got fellow rdesktop formats\n"));
+			logger(Clipboard, Debug,
+			       "xclip_handle_SelectionNotify(), got fellow rdesktop formats");
 			probing_selections = False;
 			rdesktop_is_selection_owner = True;
 			cliprdr_send_native_format_announce(data, nitems);
@@ -733,7 +773,8 @@ xclip_handle_SelectionNotify(XSelectionEvent * event)
 	xclip_clear_target_props();
 	if (probing_selections)
 	{
-		DEBUG_CLIPBOARD(("Unable to find suitable target. Using default text format.\n"));
+		logger(Clipboard, Debug,
+		       "xclip_handle_SelectionNotify(), unable to find suitable target, using default text format");
 		probing_selections = False;
 		rdesktop_is_selection_owner = False;
 
@@ -761,11 +802,17 @@ xclip_handle_SelectionRequest(XSelectionRequestEvent * event)
 	unsigned char *prop_return = NULL;
 	int format, res;
 	Atom type;
+	char *selection_name, *target_name, *property_name;
 
-	DEBUG_CLIPBOARD(("xclip_handle_SelectionRequest: selection=%s, target=%s, property=%s\n",
-			 XGetAtomName(g_display, event->selection),
-			 XGetAtomName(g_display, event->target),
-			 XGetAtomName(g_display, event->property)));
+	selection_name = XGetAtomName(g_display, event->selection);
+	target_name = XGetAtomName(g_display, event->target);
+	property_name = XGetAtomName(g_display, event->property);
+	logger(Clipboard, Debug,
+	       "xclip_handle_SelectionRequest(), selection=%s, target=%s, property=%s",
+	       selection_name, target_name, property_name);
+	XFree(selection_name);
+	XFree(target_name);
+	XFree(property_name);
 
 	if (event->target == targets_atom)
 	{
@@ -788,7 +835,8 @@ xclip_handle_SelectionRequest(XSelectionRequestEvent * event)
 		   handle one such request at a time. */
 		if (has_selection_request)
 		{
-			DEBUG_CLIPBOARD(("Error: Another clipboard request was already sent to the RDP server and not yet responded. Refusing this request.\n"));
+			logger(Clipboard, Warning,
+			       "xclip_handle_SelectionRequest(), overlapping clipboard request, skipping.");
 			xclip_refuse_selection(event);
 			return;
 		}
@@ -802,7 +850,8 @@ xclip_handle_SelectionRequest(XSelectionRequestEvent * event)
 						 &prop_return);
 			if (res != Success || (!prop_return))
 			{
-				DEBUG_CLIPBOARD(("Requested native format but didn't specifiy which.\n"));
+				logger(Clipboard, Error,
+				       "xclip_handle_SelectionRequest(), requested native format without specify which");
 				xclip_refuse_selection(event);
 				return;
 			}
@@ -820,7 +869,8 @@ xclip_handle_SelectionRequest(XSelectionRequestEvent * event)
 #ifdef USE_UNICODE_CLIPBOARD
 			format = CF_UNICODETEXT;
 #else
-			DEBUG_CLIPBOARD(("Requested target unavailable due to lack of Unicode support. (It was not in TARGETS, so why did you ask for it?!)\n"));
+			logger(Clipboard, Warning,
+			       "xclip_handle_SelectionRequest(), target unavailable due to lack of unicode support");
 			xclip_refuse_selection(event);
 			return;
 #endif
@@ -832,7 +882,11 @@ xclip_handle_SelectionRequest(XSelectionRequestEvent * event)
 		}
 		else
 		{
-			DEBUG_CLIPBOARD(("Requested target unavailable. (It was not in TARGETS, so why did you ask for it?!)\n"));
+			target_name = XGetAtomName(g_display, event->target);
+			logger(Clipboard, Warning,
+			       "xclip_handle_SelectionRequest(), unsupported target format, target='%s'",
+			       target_name);
+			XFree(target_name);
 			xclip_refuse_selection(event);
 			return;
 		}
@@ -848,13 +902,13 @@ xclip_handle_SelectionRequest(XSelectionRequestEvent * event)
    is offered by the RDP server (and when it is pasted inside RDP, there's no network
    roundtrip).
 
-   This event (SelectionClear) symbolizes this rdesktop lost onwership of the clipboard
+   This event (SelectionClear) symbolizes this rdesktop lost ownership of the clipboard
    to some other X client. We should find out what clipboard formats this other
    client offers and announce that to RDP. */
 void
 xclip_handle_SelectionClear(void)
 {
-	DEBUG_CLIPBOARD(("xclip_handle_SelectionClear\n"));
+	logger(Clipboard, Debug, "xclip_handle_SelectionClear()");
 	xclip_notify_change();
 	xclip_probe_selections();
 }
@@ -873,11 +927,11 @@ xclip_handle_PropertyNotify(XPropertyEvent * event)
 
 	if (event->state == PropertyNewValue && g_waiting_for_INCR)
 	{
-		DEBUG_CLIPBOARD(("x_clip_handle_PropertyNotify: g_waiting_for_INCR != 0\n"));
+		logger(Clipboard, Debug, "xclip_handle_PropertyNotify(), g_waiting_for_INCR != 0");
 
 		while (bytes_left > 0)
 		{
-			/* Unlike the specification, we don't set the 'delete' arugment to True
+			/* Unlike the specification, we don't set the 'delete' argument to True
 			   since we slurp the INCR's chunks in even-smaller chunks of 4096 bytes. */
 			if ((XGetWindowProperty
 			     (g_display, g_wnd, rdesktop_clipboard_target_atom, offset, 4096L,
@@ -928,7 +982,6 @@ xclip_handle_PropertyNotify(XPropertyEvent * event)
 	    (event->window == DefaultRootWindow(g_display)))
 		xclip_probe_selections();
 }
-#endif
 
 
 /* Called when the RDP server announces new clipboard data formats.
@@ -943,11 +996,11 @@ ui_clip_format_announce(uint8 * data, uint32 length)
 
 	XSetSelectionOwner(g_display, primary_atom, g_wnd, acquire_time);
 	if (XGetSelectionOwner(g_display, primary_atom) != g_wnd)
-		warning("Failed to aquire ownership of PRIMARY clipboard\n");
+		logger(Clipboard, Warning, "failed to acquire ownership of PRIMARY clipboard");
 
 	XSetSelectionOwner(g_display, clipboard_atom, g_wnd, acquire_time);
 	if (XGetSelectionOwner(g_display, clipboard_atom) != g_wnd)
-		warning("Failed to aquire ownership of CLIPBOARD clipboard\n");
+		logger(Clipboard, Warning, "failed to acquire ownership of CLIPBOARD clipboard");
 
 	if (formats_data)
 		xfree(formats_data);
@@ -1004,7 +1057,7 @@ ui_clip_handle_data(uint8 * data, uint32 length)
 				iconv_close(cd);
 				return;
 			}
-			iconv(cd, (ICONV_CONST char **) &data_remaining, &length_remaining,
+			iconv(cd, (char **) &data_remaining, &length_remaining,
 			      &utf8_data_remaining, &utf8_length_remaining);
 			iconv_close(cd);
 			free_data = True;
@@ -1027,7 +1080,11 @@ ui_clip_handle_data(uint8 * data, uint32 length)
 	}
 	else
 	{
-		DEBUG_CLIPBOARD(("ui_clip_handle_data: BUG! I don't know how to convert selection target %s!\n", XGetAtomName(g_display, selection_request.target)));
+		char *target_name = XGetAtomName(g_display, selection_request.target);
+		logger(Clipboard, Debug,
+		       "ui_clip_handle_data(), no handler for selection target '%s'",
+		       target_name);
+		XFree(target_name);
 		xclip_refuse_selection(&selection_request);
 		has_selection_request = False;
 		return;
@@ -1052,12 +1109,13 @@ ui_clip_request_data(uint32 format)
 {
 	Window primary_owner, clipboard_owner;
 
-	DEBUG_CLIPBOARD(("Request from server for format %d\n", format));
+	logger(Clipboard, Debug, "request from server for format %d", format);
 	rdp_clipboard_request_format = format;
 
 	if (probing_selections)
 	{
-		DEBUG_CLIPBOARD(("ui_clip_request_data: Selection probe in progress. Cannot handle request.\n"));
+		logger(Clipboard, Debug,
+		       "ui_clip_request_data(), selection probe in progress, cannot handle request");
 		helper_cliprdr_send_empty_response();
 		return;
 	}
@@ -1130,7 +1188,7 @@ ui_clip_set_mode(const char *optarg)
 		auto_mode = False;
 	else
 	{
-		warning("Invalid clipboard mode '%s'.\n", optarg);
+		logger(Clipboard, Warning, "invalid clipboard mode '%s'", optarg);
 		g_rdpclip = False;
 	}
 }
