@@ -4,6 +4,7 @@
 
    Copyright (C) Matthew Chapman <matthewc.unsw.edu.au> 1999-2008
    Copyright 2003-2008 Peter Astrand <astrand@cendio.se> for Cendio AB
+   Copyright 2014-2017 Henrik Andersson <hean01@cendio.se> for Cendio AB
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -19,12 +20,8 @@
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-#ifdef RDP2VNC
-#include "vnc/x11stubs.h"
-#else
 #include <X11/Xlib.h>
 #include <X11/keysym.h>
-#endif
 
 #include <ctype.h>
 #include <limits.h>
@@ -34,8 +31,8 @@
 #include "rdesktop.h"
 #include "scancodes.h"
 
-#define KEYMAP_SIZE 0xffff+1
-#define KEYMAP_MASK 0xffff
+#define KEYMAP_SIZE 0x7f+1
+#define KEYMAP_MASK 0x7f
 #define KEYMAP_MAX_LINE_LENGTH 80
 
 extern Display *g_display;
@@ -47,11 +44,13 @@ extern int g_keyboard_subtype;
 extern int g_keyboard_functionkeys;
 extern int g_win_button_size;
 extern RD_BOOL g_enable_compose;
-extern RD_BOOL g_use_rdp5;
+extern RD_BOOL g_seamless_rdp;
+extern RD_BOOL g_seamless_active;
+extern RDP_VERSION g_rdp_version;
 extern RD_BOOL g_numlock_sync;
 
 static RD_BOOL keymap_loaded;
-static key_translation *keymap[KEYMAP_SIZE];
+static key_translation_entry *keymap[KEYMAP_SIZE];
 static KeySym keypress_keysyms[256];
 static int min_keycode;
 static uint16 remote_modifier_state = 0;
@@ -73,28 +72,113 @@ free_key_translation(key_translation * ptr)
 	}
 }
 
+/* Free the key_translation_entry for a given keysym and remove from the table */
+static void
+delete_key_translation_entry(KeySym keysym)
+{
+	uint32 hash;
+	key_translation_entry *ptr;
+	key_translation_entry *next;
+	key_translation_entry *prev;
+	key_translation_entry tmp;
+
+	/* Faking a prev node allows us to keep the algorithm simple */
+	hash = keysym & KEYMAP_MASK;
+	ptr = keymap[hash];
+	tmp.next = ptr;
+	prev = &tmp;
+
+	while (ptr)
+	{
+		next = ptr->next;
+		if (ptr->keysym == keysym)
+		{
+			free_key_translation(ptr->tr);
+			prev->next = next;
+			xfree(ptr);
+		}
+		else
+		{
+			prev = ptr;
+		}
+
+		ptr = next;
+	}
+
+	/* Copy pointer back from our fake node */
+	keymap[hash] = tmp.next;
+}
+
+/* Allocate and return a new entry in the translation table */
+static key_translation_entry *
+new_key_translation_entry(KeySym keysym)
+{
+	uint32 hash;
+	key_translation_entry *entry;
+
+	/* Clear out any existing entry */
+	delete_key_translation_entry(keysym);
+
+	/* Allocate the new one */
+	entry = (key_translation_entry *) xmalloc(sizeof(key_translation_entry));
+	memset(entry, 0, sizeof(key_translation_entry));
+	entry->keysym = keysym;
+
+	/* And insert it at head of list */
+	hash = keysym & KEYMAP_MASK;
+	entry->next = keymap[hash];
+	keymap[hash] = entry;
+
+	return entry;
+}
+
+/* Retrieve the key_translation_entry for a given keysym */
+static key_translation_entry *
+get_key_translation_entry(uint32 keysym)
+{
+	key_translation_entry *ptr;
+	key_translation_entry *next;
+
+	ptr = keymap[keysym & KEYMAP_MASK];
+
+	while (ptr)
+	{
+		next = ptr->next;
+		if (ptr->keysym == keysym)
+			return ptr;
+
+		ptr = next;
+	}
+
+	/* Not found */
+	return NULL;
+}
+
 static void
 add_to_keymap(char *keyname, uint8 scancode, uint16 modifiers, char *mapname)
 {
 	KeySym keysym;
-	key_translation *tr;
+	key_translation_entry *entry;
 
 	keysym = XStringToKeysym(keyname);
 	if (keysym == NoSymbol)
 	{
-		DEBUG_KBD(("Bad keysym \"%s\" in keymap %s (ignoring)\n", keyname, mapname));
+		logger(Keyboard, Error, "add_to_keymap(), ignoring bad keysym '%s' in keymap %s",
+		       keyname, mapname);
 		return;
 	}
 
-	DEBUG_KBD(("Adding translation, keysym=0x%x, scancode=0x%x, "
-		   "modifiers=0x%x\n", (unsigned int) keysym, scancode, modifiers));
+	logger(Keyboard, Debug, "add_to_keymap(), adding translation, keysym=0x%x, scancode=0x%x, "
+	       "modifiers=0x%x", (unsigned int) keysym, scancode, modifiers);
 
-	tr = (key_translation *) xmalloc(sizeof(key_translation));
-	memset(tr, 0, sizeof(key_translation));
-	tr->scancode = scancode;
-	tr->modifiers = modifiers;
-	free_key_translation(keymap[keysym & KEYMAP_MASK]);
-	keymap[keysym & KEYMAP_MASK] = tr;
+	/* Make a new entry in the table */
+	entry = new_key_translation_entry(keysym);
+
+	/* And add the new translation to it */
+	entry->tr = (key_translation *) xmalloc(sizeof(key_translation));
+	memset(entry->tr, 0, sizeof(key_translation));
+	entry->tr->scancode = scancode;
+	entry->tr->modifiers = modifiers;
 
 	return;
 }
@@ -103,6 +187,8 @@ static void
 add_sequence(char *rest, char *mapname)
 {
 	KeySym keysym;
+	KeySym seq_keysym;
+	key_translation_entry *entry;
 	key_translation *tr, **prev_next;
 	size_t chars;
 	char keyname[KEYMAP_MAX_LINE_LENGTH];
@@ -119,15 +205,16 @@ add_sequence(char *rest, char *mapname)
 	keysym = XStringToKeysym(keyname);
 	if (keysym == NoSymbol)
 	{
-		DEBUG_KBD(("Bad keysym \"%s\" in keymap %s (ignoring line)\n", keyname, mapname));
+		logger(Keyboard, Error, "add_sequence(), ignoring bad keysym '%s' in keymap %s",
+		       keyname, mapname);
 		return;
 	}
 
+	logger(Keyboard, Debug, "add_sequence(), adding sequence for keysym '%s' (0x%lx)", keyname,
+	       keysym);
 
-	DEBUG_KBD(("Adding sequence for keysym (0x%lx, %s) -> ", keysym, keyname));
-
-	free_key_translation(keymap[keysym & KEYMAP_MASK]);
-	prev_next = &keymap[keysym & KEYMAP_MASK];
+	entry = new_key_translation_entry(keysym);
+	prev_next = &(entry->tr);
 
 	while (*rest)
 	{
@@ -140,24 +227,32 @@ add_sequence(char *rest, char *mapname)
 		STRNCPY(keyname, rest, chars + 1);
 		rest += chars;
 
-		keysym = XStringToKeysym(keyname);
-		if (keysym == NoSymbol)
+		/* Handle trailing whitespace */
+		if (*keyname == 0)
+			break;
+
+		seq_keysym = XStringToKeysym(keyname);
+		if (seq_keysym == NoSymbol)
 		{
-			DEBUG_KBD(("Bad keysym \"%s\" in keymap %s (ignoring line)\n", keyname,
-				   mapname));
+			logger(Keyboard, Error,
+			       "add_sequence(), ignoring line with bad keysym '%s' in keymap %s",
+			       keyname, mapname);
+			delete_key_translation_entry(keysym);
 			return;
 		}
 
 		/* Allocate space for key_translation structure */
 		tr = (key_translation *) xmalloc(sizeof(key_translation));
 		memset(tr, 0, sizeof(key_translation));
+
+		/* Do this straight away so the key_translation won't get orphaned on error */
+		if (!entry->tr)
+			entry->tr = tr;
+
 		*prev_next = tr;
 		prev_next = &tr->next;
-		tr->seq_keysym = keysym;
-
-		DEBUG_KBD(("0x%x, ", (unsigned int) keysym));
+		tr->seq_keysym = seq_keysym;
 	}
-	DEBUG_KBD(("\n"));
 }
 
 RD_BOOL
@@ -291,7 +386,7 @@ xkeymap_read(char *mapname)
 	fp = xkeymap_open(mapname);
 	if (fp == NULL)
 	{
-		error("Failed to open keymap %s\n", mapname);
+		logger(Keyboard, Error, "xkeymap_read(), failed to open keymap %s", mapname);
 		return False;
 	}
 
@@ -325,14 +420,14 @@ xkeymap_read(char *mapname)
 		if (str_startswith(line, "map "))
 		{
 			g_keylayout = strtoul(line + sizeof("map ") - 1, NULL, 16);
-			DEBUG_KBD(("Keylayout 0x%x\n", g_keylayout));
+			logger(Keyboard, Debug, "xkeymap_read(), Keylayout 0x%x", g_keylayout);
 			continue;
 		}
 
 		/* compose */
 		if (str_startswith(line, "enable_compose"))
 		{
-			DEBUG_KBD(("Enabling compose handling\n"));
+			logger(Keyboard, Debug, "xkeymap_read(), enabling compose handling");
 			g_enable_compose = True;
 			continue;
 		}
@@ -348,7 +443,8 @@ xkeymap_read(char *mapname)
 		if (str_startswith(line, "keyboard_type "))
 		{
 			g_keyboard_type = strtol(line + sizeof("keyboard_type ") - 1, NULL, 16);
-			DEBUG_KBD(("keyboard_type 0x%x\n", g_keyboard_type));
+			logger(Keyboard, Debug, "xkeymap_read(), keyboard_type 0x%x",
+			       g_keyboard_type);
 			continue;
 		}
 
@@ -357,7 +453,8 @@ xkeymap_read(char *mapname)
 		{
 			g_keyboard_subtype =
 				strtol(line + sizeof("keyboard_subtype ") - 1, NULL, 16);
-			DEBUG_KBD(("keyboard_subtype 0x%x\n", g_keyboard_subtype));
+			logger(Keyboard, Debug, "xkeymap_read(), keyboard_subtype 0x%x",
+			       g_keyboard_subtype);
 			continue;
 		}
 
@@ -366,7 +463,8 @@ xkeymap_read(char *mapname)
 		{
 			g_keyboard_functionkeys =
 				strtol(line + sizeof("keyboard_functionkeys ") - 1, NULL, 16);
-			DEBUG_KBD(("keyboard_functionkeys 0x%x\n", g_keyboard_functionkeys));
+			logger(Keyboard, Debug, "xkeymap_read(), keyboard_functionkeys 0x%x",
+			       g_keyboard_functionkeys);
 			continue;
 		}
 
@@ -381,7 +479,8 @@ xkeymap_read(char *mapname)
 		p = strchr(line, ' ');
 		if (p == NULL)
 		{
-			error("Bad line %d in keymap %s\n", line_num, mapname);
+			logger(Keyboard, Error, "xkeymap_read(), bad line %d in keymap %s",
+			       line_num, mapname);
 			continue;
 		}
 		else
@@ -467,7 +566,7 @@ send_winkey(uint32 ev_time, RD_BOOL pressed, RD_BOOL leftkey)
 
 	if (pressed)
 	{
-		if (g_use_rdp5)
+		if (g_rdp_version >= RDP_V5)
 		{
 			rdp_send_scancode(ev_time, RDP_KEYPRESS, winkey);
 		}
@@ -481,7 +580,7 @@ send_winkey(uint32 ev_time, RD_BOOL pressed, RD_BOOL leftkey)
 	else
 	{
 		/* key released */
-		if (g_use_rdp5)
+		if (g_rdp_version >= RDP_V5)
 		{
 			rdp_send_scancode(ev_time, RDP_KEYRELEASE, winkey);
 		}
@@ -496,7 +595,7 @@ send_winkey(uint32 ev_time, RD_BOOL pressed, RD_BOOL leftkey)
 static void
 reset_winkey(uint32 ev_time)
 {
-	if (g_use_rdp5)
+	if (g_rdp_version >= RDP_V5)
 	{
 		/* For some reason, it seems to suffice to release
 		 *either* the left or right winkey. */
@@ -547,7 +646,21 @@ handle_special_keys(uint32 keysym, unsigned int state, uint32 ev_time, RD_BOOL p
 			{
 				/* Ctrl-Alt-Enter: toggle full screen */
 				if (pressed)
-					xwin_toggle_fullscreen();
+				{
+					if (!g_seamless_rdp)
+					{
+						/* only allow toggle fullscreen when not running
+						   rdesktop in seamless mode */
+						xwin_toggle_fullscreen();
+					}
+					else
+					{
+						/* deactivate seamless mode for debug purpose, one can
+						   not activate seamless mode again  */
+						if (g_seamless_active)
+							ui_seamless_toggle();
+					}
+				}
 				return True;
 			}
 			break;
@@ -566,28 +679,22 @@ handle_special_keys(uint32 keysym, unsigned int state, uint32 ev_time, RD_BOOL p
 			break;
 
 		case XK_Pause:
-			/* According to MS Keyboard Scan Code
-			   Specification, pressing Pause should result
-			   in E1 1D 45 E1 9D C5. I'm not exactly sure
-			   of how this is supposed to be sent via
-			   RDP. The code below seems to work, but with
-			   the side effect that Left Ctrl stays
-			   down. Therefore, we release it when Pause
-			   is released. */
+			/* According to the RDP documentation (MS-RDPBCGR, page 164),
+			   pressing Pause must result in:
+			   CTRL    (0x1D) DOWN  with the KBDFLAGS_EXTENDED1 flag
+			   NUMLOCK (0x45) DOWN
+			   CTRL    (0x1D) UP    with the KBDFLAGS_EXTENDED1 flag
+			   NUMLOCK (0x45) UP
+			 */
 			if (pressed)
 			{
-				rdp_send_input(ev_time, RDP_INPUT_SCANCODE, RDP_KEYPRESS, 0xe1, 0);
-				rdp_send_input(ev_time, RDP_INPUT_SCANCODE, RDP_KEYPRESS, 0x1d, 0);
+				rdp_send_input(ev_time, RDP_INPUT_SCANCODE,
+					       RDP_KEYPRESS | KBD_FLAG_EXT1, 0x1d, 0);
 				rdp_send_input(ev_time, RDP_INPUT_SCANCODE, RDP_KEYPRESS, 0x45, 0);
-				rdp_send_input(ev_time, RDP_INPUT_SCANCODE, RDP_KEYPRESS, 0xe1, 0);
-				rdp_send_input(ev_time, RDP_INPUT_SCANCODE, RDP_KEYPRESS, 0x9d, 0);
-				rdp_send_input(ev_time, RDP_INPUT_SCANCODE, RDP_KEYPRESS, 0xc5, 0);
-			}
-			else
-			{
-				/* Release Left Ctrl */
-				rdp_send_input(ev_time, RDP_INPUT_SCANCODE, RDP_KEYRELEASE,
-					       0x1d, 0);
+				rdp_send_input(ev_time, RDP_INPUT_SCANCODE,
+					       RDP_KEYRELEASE | KBD_FLAG_EXT1, 0x1d, 0);
+				rdp_send_input(ev_time, RDP_INPUT_SCANCODE, RDP_KEYRELEASE, 0x45,
+					       0);
 			}
 			return True;
 			break;
@@ -638,8 +745,11 @@ xkeymap_translate_key(uint32 keysym, unsigned int keycode, unsigned int state)
 {
 	key_translation tr = { 0, 0, 0, 0 };
 	key_translation *ptr;
+	key_translation_entry *entry;
+
+	entry = get_key_translation_entry(keysym);
+	ptr = entry ? entry->tr : NULL;
 
-	ptr = keymap[keysym & KEYMAP_MASK];
 	if (ptr)
 	{
 		tr = *ptr;
@@ -647,7 +757,7 @@ xkeymap_translate_key(uint32 keysym, unsigned int keycode, unsigned int state)
 		{
 			if (MASK_HAS_BITS(tr.modifiers, MapInhibitMask))
 			{
-				DEBUG_KBD(("Inhibiting key\n"));
+				logger(Keyboard, Debug, "xkeymap_translate_key(), inhibiting key");
 				tr.scancode = 0;
 				return tr;
 			}
@@ -672,19 +782,21 @@ xkeymap_translate_key(uint32 keysym, unsigned int keycode, unsigned int state)
 			    && MASK_HAS_BITS(remote_modifier_state, MapCtrlMask)
 			    && !MASK_HAS_BITS(state, ShiftMask))
 			{
-				DEBUG_KBD(("Non-physical Shift + Ctrl pressed, releasing Shift\n"));
+				logger(Keyboard, Debug,
+				       "xkeymap_translate_key(), non-physical Shift + Ctrl pressed, releasing Shift");
 				MASK_REMOVE_BITS(tr.modifiers, MapShiftMask);
 			}
 
-			DEBUG_KBD(("Found scancode translation, scancode=0x%x, modifiers=0x%x\n",
-				   tr.scancode, tr.modifiers));
+			logger(Keyboard, Debug,
+			       "xkeymap_translate_key(), found scancode translation, scancode=0x%x, modifiers=0x%x",
+			       tr.scancode, tr.modifiers);
 		}
 	}
 	else
 	{
 		if (keymap_loaded)
-			warning("No translation for (keysym 0x%lx, %s)\n", keysym,
-				get_ksname(keysym));
+			logger(Keyboard, Warning, "No translation for (keysym 0x%lx, %s)", keysym,
+			       get_ksname(keysym));
 
 		/* not in keymap, try to interpret the raw scancode */
 		if (((int) keycode >= min_keycode) && (keycode <= 0x60))
@@ -699,11 +811,11 @@ xkeymap_translate_key(uint32 keysym, unsigned int keycode, unsigned int state)
 				tr.modifiers = MapLeftShiftMask;
 			}
 
-			DEBUG_KBD(("Sending guessed scancode 0x%x\n", tr.scancode));
+			logger(Keyboard, Debug, "Sending guessed scancode 0x%x", tr.scancode);
 		}
 		else
 		{
-			DEBUG_KBD(("No good guess for keycode 0x%x found\n", keycode));
+			logger(Keyboard, Debug, "No good guess for keycode 0x%x found", keycode);
 		}
 	}
 
@@ -758,12 +870,13 @@ xkeymap_send_keys(uint32 keysym, unsigned int keycode, unsigned int state, uint3
 		ptr = &tr;
 		do
 		{
-			DEBUG_KBD(("Handling sequence element, keysym=0x%x\n",
-				   (unsigned int) ptr->seq_keysym));
+			logger(Keyboard, Debug,
+			       "xkeymap_send_keys(), handling sequence element, keysym=0x%x",
+			       (unsigned int) ptr->seq_keysym);
 
 			if (nesting++ > 32)
 			{
-				error("Sequence nesting too deep\n");
+				logger(Keyboard, Error, "Sequence nesting too deep");
 				return;
 			}
 
@@ -776,8 +889,10 @@ xkeymap_send_keys(uint32 keysym, unsigned int keycode, unsigned int state, uint3
 }
 
 uint16
-xkeymap_translate_button(unsigned int button)
+xkeymap_translate_button(unsigned int button, uint16 * input_type)
 {
+	*input_type = RDP_INPUT_MOUSE;
+
 	switch (button)
 	{
 		case Button1:	/* left */
@@ -790,6 +905,12 @@ xkeymap_translate_button(unsigned int button)
 			return MOUSE_FLAG_BUTTON4;
 		case Button5:	/* wheel down */
 			return MOUSE_FLAG_BUTTON5;
+		case 8:	/* button 4 */
+			*input_type = RDP_INPUT_MOUSEX;
+			return MOUSEX_FLAG_BUTTON1;
+		case 9:	/* button 5 */
+			*input_type = RDP_INPUT_MOUSEX;
+			return MOUSEX_FLAG_BUTTON2;
 	}
 
 	return 0;
@@ -820,7 +941,7 @@ save_remote_modifiers(uint8 scancode)
 void
 restore_remote_modifiers(uint32 ev_time, uint8 scancode)
 {
-	key_translation dummy = { };
+	key_translation dummy = { 0 };
 
 	if (is_modifier(scancode))
 		return;
@@ -848,13 +969,15 @@ ensure_remote_modifiers(uint32 ev_time, key_translation tr)
 
 			if (MASK_HAS_BITS(tr.modifiers, MapNumLockMask))
 			{
-				DEBUG_KBD(("Remote NumLock state is incorrect, activating NumLock.\n"));
+				logger(Keyboard, Debug,
+				       "ensure_remote_modifiers(), remote NumLock state is incorrect, activating NumLock");
 				new_remote_state = KBD_FLAG_NUMLOCK;
 				remote_modifier_state = MapNumLockMask;
 			}
 			else
 			{
-				DEBUG_KBD(("Remote NumLock state is incorrect, deactivating NumLock.\n"));
+				logger(Keyboard, Debug,
+				       "ensure_remote_modifiers(), remote NumLock state is incorrect, deactivating NumLock.");
 				new_remote_state = 0;
 				remote_modifier_state = 0;
 			}
@@ -915,16 +1038,12 @@ ensure_remote_modifiers(uint32 ev_time, key_translation tr)
 unsigned int
 read_keyboard_state()
 {
-#ifdef RDP2VNC
-	return 0;
-#else
 	unsigned int state;
 	Window wdummy;
 	int dummy;
 
 	XQueryPointer(g_display, g_wnd, &wdummy, &wdummy, &dummy, &dummy, &dummy, &dummy, &state);
 	return state;
-#endif
 }
 
 
@@ -983,11 +1102,9 @@ reset_modifier_keys()
 static void
 update_modifier_state(uint8 scancode, RD_BOOL pressed)
 {
-#ifdef WITH_DEBUG_KBD
 	uint16 old_modifier_state;
 
 	old_modifier_state = remote_modifier_state;
-#endif
 
 	switch (scancode)
 	{
@@ -1029,15 +1146,14 @@ update_modifier_state(uint8 scancode, RD_BOOL pressed)
 			}
 	}
 
-#ifdef WITH_DEBUG_KBD
 	if (old_modifier_state != remote_modifier_state)
 	{
-		DEBUG_KBD(("Before updating modifier_state:0x%x, pressed=0x%x\n",
-			   old_modifier_state, pressed));
-		DEBUG_KBD(("After updating modifier_state:0x%x\n", remote_modifier_state));
+		logger(Keyboard, Debug,
+		       "update_modifier_state(), before modifier_state:0x%x, pressed=0x%x",
+		       old_modifier_state, pressed);
+		logger(Keyboard, Debug, "update_modifier_state(), after modifier_state:0x%x",
+		       remote_modifier_state);
 	}
-#endif
-
 }
 
 /* Send keyboard input */
@@ -1048,14 +1164,16 @@ rdp_send_scancode(uint32 time, uint16 flags, uint8 scancode)
 
 	if (scancode & SCANCODE_EXTENDED)
 	{
-		DEBUG_KBD(("Sending extended scancode=0x%x, flags=0x%x\n",
-			   scancode & ~SCANCODE_EXTENDED, flags));
+		logger(Keyboard, Debug,
+		       "rdp_send_scancode(), sending extended scancode=0x%x, flags=0x%x",
+		       scancode & ~SCANCODE_EXTENDED, flags);
 		rdp_send_input(time, RDP_INPUT_SCANCODE, flags | KBD_FLAG_EXT,
 			       scancode & ~SCANCODE_EXTENDED, 0);
 	}
 	else
 	{
-		DEBUG_KBD(("Sending scancode=0x%x, flags=0x%x\n", scancode, flags));
+		logger(Keyboard, Debug, "rdp_send_scancode(), sending scancode=0x%x, flags=0x%x",
+		       scancode, flags);
 		rdp_send_input(time, RDP_INPUT_SCANCODE, flags, scancode, 0);
 	}
 }
